import os
import sys
from unicodedata import normalize

_windows_device_files = ('CON', 'AUX', 'COM1', 'COM2', 'COM3', 'COM4', 'LPT1',
'LPT2', 'LPT3', 'PRN', 'NUL')

def secure_filename(filename, codec='utf8'):
    r"""Pass it a filename and it will return a secure version of it.

    This finction is a modified version of |werkzeug-secure_filename|_.

    .. |werkzeug-secure_filename| replace:: ``werkzeug.utils.secure_filename``
    .. _werkzeug-secure_filename: http://werkzeug.pocoo.org/docs/0.9/utils/#werkzeug.utils.secure_filename

    The filename can then safely be stored on a regular file system and passed
    to :func:`os.path.join`.

    You can use parameter ``codec`` to specify the codec which is used
    to encode the filename. The ``codec`` could only be *utf8* or *ascii*.
    If you need high portability, you should let ``codec`` to be ``'ascii'``.
    It will be ``'utf8'`` by default.

    On windows systems the function also makes sure that the file is not
    named after one of the special device files.

    >>> secure_filename("My cool movie.mov")
    'My_cool_movie.mov'
    >>> secure_filename("../../../etc/passwd")
    'etc_passwd'
    >>> secure_filename('i contain cool \xfcml\xe4uts.txt')
    'i_contain_cool_ümläuts.txt'
    >>> secure_filename('i contain cool \xfcml\xe4uts.txt', 'ascii')
    'i_contain_cool_umlauts.txt'

    The function might return an empty filename. It's your responsibility
    to ensure that the filename is unique and that you generate random
    filename if the function returned an empty one.

    :param filename: the filename to secure
    """
    if codec.lower() not in ('utf8', 'utf-8', 'ascii'):
        raise ValueError('Argument ``codec`` should be *utf8* or *ascii*.')

    if isinstance(filename, str):
        filename, ext = os.path.splitext(filename)
        filename = normalize('NFKD', filename).encode(codec, 'ignore').decode(codec)
    else:
        raise TypeError('Filename should be a instance of str.')

    for sep in os.path.sep, os.path.altsep:
        if sep:
            filename = filename.replace(sep, ' ')
    filename = '_'.join(filename.split()) + ext
    filename = filename.strip('._')

    if os.name == 'nt' and filename and filename.split('.')[0].upper() in _windows_device_files:
        filename = '_' + filename

    return filename



import hashlib
import random
import string
import time

try:
    random = random.SystemRandom()
    using_sysrandom = True
except NotImplementedError:
    using_sysrandom = False

def get_random_string(length=36, allowed_chars='complex'):
    """Returns a securely generated random string.

    Generate a securely random string with length or allowed
    characters you want.

    If *allowed_chars* is ignored, it's ``string.ascii_letters + 
    string.digits + string.punctuation`` by default (We use Python
    builtin library `string`_ to set value). You can set it to following
    string values or any *allowed_chars* else.

    .. _`string`: http://docs.python.org/3/library/string.html

    .. tabularcolumns:: |l|L|

    =============== =========================
          Value               Meaning
    =============== =========================
    ``'complex'``     string.ascii_letters + 
                      string.digits + 
                      string.punctuation
    --------------- -------------------------
    ``'middle'``      string.ascii_letters + 
                      string.digits + 
                      ``'-._~'``
    --------------- -------------------------
    ``'simple'``      string.ascii_letters +
                      string.digits
    --------------- -------------------------
    ``'lowercase'``   string.ascii_lowercase
    --------------- -------------------------
    ``'uppercase'``   string.ascii_uppercase
    --------------- -------------------------
    ``'digits'``      string.digits
    =============== =========================

    >>> get_random_string(8) # doctest: +SKIP
    'Qi4Y_fA5'

    >>> get_random_string(5, '0123456789abcdef') # doctest: +SKIP
    '06aef'

    >>> get_random_string(5, 'digits') # doctest: +SKIP
    '05154'

    :param length: Random string length.
    :param allowed_chars: Character allowed to use to generate.
        There has 5 default setting (case-insensitive).
    :type length: int
    :type allowed_chars: str
    :return: A securely generated random string.
    :rtype: str
    """
    case = {
        'complex': string.ascii_letters + string.digits + string.punctuation,
        'middle': string.ascii_letters + string.digits + '-._~',
        'simple': string.ascii_letters + string.digits,
        'lowercase': string.ascii_lowercase,
        'uppercase': string.ascii_uppercase,
        'digits': string.digits
    }

    allowed_chars = case.get(allowed_chars.lower() or 'complex', allowed_chars)

    if not using_sysrandom:
        random.seed(
            hashlib.sha256(
                ("{}{}".format(
                    time.time(),
                    random.getstate(),
                    )).encode('utf-8')
            ).digest())
    return ''.join(random.choice(allowed_chars) for _ in range(length))



import re
import ast
from setuptools import setup


_version_re = re.compile(r'__version__\s+=\s+(.*)')


with open('NIWLittleUtils/__init__.py', 'rb') as f:
    version = ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')).group(1))

with open('README.rst', 'rb') as f:
    long_description = f.read().decode('utf-8')

setup(
    name='NIWLittleUtils',
    author='NIWyclin (a.k.a. birdhackor)',
    author_email='NIWLittleUtils@mymail.niwyclin.org',
    license='LGPLv3',
    version=version,
    url='https://bitbucket.org/birdhackor/niwlittleutils',
    packages=['NIWLittleUtils'],
    description='NIWLittleUtils is a tool collection.',
    long_description=long_description,
    classifiers=[
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
)



import base64
import zlib

def b64encode(b):
    """Base64 encodes a byte string.

    Base64 encodes a byte string. The resulting bytestring
    is safe for putting into URLs. The input ``b`` should be
    a ``bytes`` or ``bytearray`` object in Python3.3 or
    any `bytes-like object`_ after Python3.4. This function
    will raise ``TypeError`` if ``b`` is not
    `bytes-like object`_ (For example, ``str``).

    .. _`bytes-like object`: https://docs.python.org/3/glossary.html#term-bytes-like-object

    :rtype: bytes
    """
    return base64.urlsafe_b64encode(b).strip(b'=')

def b64decode(b):
    """Decode a Base64 encoded byte string.

    Decode a Base64 encoded byte string using a URL-safe
    alphabet, which substitutes ``-`` instead of ``+``
    and ``_`` instead of ``/`` in the standard Base64
    alphabet. The input ``b`` may also be a ASCII-only
    Unicode strings.

    :rtype: bytes
    """
    if isinstance(b, str):
        b = b.encode('ascii')
    return base64.urlsafe_b64decode(b + b'=' * (-len(b) % 4))

def urlsafe_encode(s):
    """Encode a string or byte string to url-safe string.

    Encode a string or byte string to url-safe string.
    The input can be a object of ``str``, ``bytes`` or
    ``bytearray``.

    This function will make sure that return string
    contains characters that *RFC 3986 section 2.3*
    specified only, but it won't check the length.
    User should remember that browsers have limit for
    length of URLs. (For example, IE8's maximum URL
    length is 2083 chars)

    :rtype: str
    """
    is_compressed = False

    if isinstance(s, str):
        is_str = True
        b = s.encode(errors='surrogateescape')
    else:
        is_str = False
        b = s

    compressed = zlib.compress(b, 9)
    if len(compressed) < (len(b) - 1):
        b = compressed
        is_compressed = True
    b64ed = b64encode(b)
    if is_str:
        b64ed = b'~' + b64ed
    if is_compressed:
        b64ed = b'.' + b64ed
    return b64ed.decode()

def urlsafe_decode(s):
    """Decode a string that encode by :func:`~NIWLittleUtils.codec.urlsafe_encode`.

    Decode a string that encoded by
    :func:`~NIWLittleUtils.codec.urlsafe_encode`.
    If the input ``s`` which returned by
    :func:`~NIWLittleUtils.codec.urlsafe_encode`
    is encode from ``bytes`` or ``bytearray``
    object, it will return ``bytes`` object.
    If ``s`` is encode from ``str`` object, it
    will return ``str`` object.

    :rtype: str or bytes
    """
    s = s.encode()

    is_compressed = False
    is_str = False
    if s.startswith(b'.'):
        s = s[1:]
        is_compressed = True
    if s.startswith(b'~'):
        s = s[1:]
        is_str = True

    try:
        b = b64decode(s)
    except Exception as e:
        raise ValueError('Could not base64 decode '
                         'because of an exception: '
                         '{}'.format(e))
    if is_compressed:
        try:
            b = zlib.decompress(b)
        except Exception as e:
            raise ValueError('Could not zlib '
                             'decompress because '
                             'of an exception: '
                             '{}'.format(e))

    return b.decode(errors='surrogateescape') if is_str else b
